<?php

/**
 * @file
 * Provides OneHourTranslation translation plugin controller.
 */

/**
 * OHT translation plugin controller.
 */
class TMGMTOhtPluginController extends TMGMTDefaultTranslatorPluginController {

  protected $ohtLanguagesMapping = array(
    'af' => 'af',
    'ar' => 'ar-sa',
    'az' => 'az-az',
    'bg' => 'bg-bg',
    'bn' => 'bn-bd',
    'bs' => 'bs-ba',
    'ca' => 'ca-es',
    'cs' => 'cs-cz',
    'da' => 'da',
    'de' => 'de-de',
    'el' => 'el-gr',
    'en' => 'en-us',
    // 'es-ar' => 'es-ar', // Not supported in Drupal.
    'es' => 'es-es',
    'et' => 'et-ee',
    // 'fa' => 'fa-af', // Not supported in Drupal.
    'fa' => 'fa-ir',
    'fi' => 'fi-fi',
    // 'fl-be' => 'fl-be', // Not supported in Drupal.
    // 'fl-fl' => 'fl-fl', // Not supported in Drupal.
    'fr' => 'fr-fr',
    // 'fr' => 'fr-ca', // Not supported in Drupal.
    'gu' => 'gu-in',
    'he' => 'he-il',
    'hi' => 'hi-in',
    'hr' => 'hr-hr',
    'ht' => 'ht',
    'hu' => 'hu-hu',
    'hy' => 'hy-am',
    'id' => 'id-id',
    'is' => 'is-is',
    'it' => 'it-it',
    'ja' => 'ja-jp',
    'ka' => 'ka-ge',
    'kk' => 'kk-kz',
    'km' => 'km-kh',
    'ko' => 'ko-kp',
    'ku' => 'ku-tr',
    'lt' => 'lt-lt',
    'lv' => 'lv-lv',
    'mk' => 'mk-mk',
    'mr' => 'mr-in',
    'ms' => 'ms-my',
    'nl' => 'nl-nl',
    'nb' => 'no-no', // We are using Norwegian Bokmål.
    'pa' => 'pa-in',
    'pl' => 'pl-pl',
    'ps' => 'ps',
    'pt' => 'pt-pt',
    'pt-br' => 'pt-br',
    'pt-pt' => 'pt-pt',
    'ro' => 'ro-ro',
    'ru' => 'ru-ru',
    'sa' => 'sa-in',
    'sk' => 'sk-sk',
    'sl' => 'sl-si',
    'sq' => 'sq-al',
    'sr' => 'sr-rs',
    'sv' => 'sv-se',
    'ta' => 'ta-in',
    'th' => 'th-th',
    'tl' => 'tl-ph',
    'tr' => 'tr-tr',
    'uk' => 'uk-ua',
    'ur' => 'ur-pk',
    'uz' => 'uz-uz',
    'vi' => 'vi-vn',
    'zh-hans' => 'zh-cn-cmn-s',
    'zh-hant' => 'zh-cn-cmn-t',
    // 'zh-cn-yue' => 'zh-cn-yue',  // Not supported in Drupal.
  );

  /**
   * Implements TMGMTTranslatorPluginControllerInterface::getDefaultRemoteLanguagesMappings().
   */
  public function getDefaultRemoteLanguagesMappings() {
    return $this->ohtLanguagesMapping;
  }

  /**
   * Implements TMGMTTranslatorPluginControllerInterface::getSupportedLanguages().
   */
  public function getSupportedRemoteLanguages(TMGMTTranslator $translator) {
    if (!empty($this->supportedRemoteLanguages)) {
      return $this->supportedRemoteLanguages;
    }

    try {
      $result = $this->getConnector($translator)->getSupportedLanguages();
      $result = json_decode($result, TRUE);

      // Parse languages.
      if (isset($result['results'])) {
        foreach ($result['results'] as $language) {
          $this->supportedRemoteLanguages[$language['code']] = $language['code'];
        }
      }
    } catch (TMGMTOHTException $e) {
      return array();
    }
    return $this->supportedRemoteLanguages;
  }

  /**
   * Implements TMGMTTranslatorPluginControllerInterface::getSupportedTargetLanguages().
   */
  public function getSupportedTargetLanguages(TMGMTTranslator $translator, $source_language) {
    $results = array();
    $language_pairs = $translator->getSupportedLanguagePairs();
    foreach ($language_pairs as $language_pair) {
      if ($source_language == $translator->mapToLocalLanguage($language_pair['source_language'])) {
        $target_language = $translator->mapToLocalLanguage($language_pair['target_language']);
        $results[$target_language] = $target_language;
      }
    }

    return $results;
  }

  /**
   * Implements TMGMTTranslatorPluginControllerInterface::getSupportedLanguagePairs().
   */
  public function getSupportedLanguagePairs(TMGMTTranslator $translator) {
    $language_pairs = array();

    try {
      $result = $this->getConnector($translator)->getSupportedLanguagePairs();
      $result = json_decode($result, TRUE);

      // Build a mapping of source and target language pairs.
      foreach ($result['results'] as $language) {
        foreach ($language['targets'] as $target_language) {
          $language_pairs[] = array('source_language' => $language['source']['code'], 'target_language' => $target_language['code']);
        }
      }
    } catch (TMGMTOHTException $e) {
      return array();
    }

    return $language_pairs;
  }

  /**
   * Implements TMGMTTranslatorPluginControllerInterface::isAvailable().
   */
  public function isAvailable(TMGMTTranslator $translator) {
    if ($translator->getSetting('api_public_key') && $translator->getSetting('api_secret_key')) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Implements TMGMTTranslatorPluginControllerInterface::requestTranslation().
   *
   * Here we will acutally query source and get translations.
   */
  public function requestTranslation(TMGMTJob $job) {
    $translator = $job->getTranslator();
    $xlf_converter = tmgmt_file_format_controller('xlf');

    /**
     * @var TMGMTJobItem $job_item
     */
    $job_item = NULL;
    $oht_uuid = NULL;

    try {

      foreach ($job->getItems() as $job_item) {
        $conditions = array('tjiid' => array('value' => $job_item->tjiid));
        $xlf = $xlf_converter->export($job, $conditions);
        $name = "JobID" . $job->tjid . '_JobItemID' . $job_item->tjiid . '_' . $job->source_language . '_' . $job->target_language;
        $oht_uuid = $this->getConnector($translator)->uploadFileResource($xlf, $name);
        $oht_uuid = array_shift($oht_uuid);

        $result = $this->getConnector($translator)->newTranslationProject(
          $job_item->tjiid,
          $this->mapToRemoteLanguage($translator, $job->source_language),
          $this->mapToRemoteLanguage($translator, $job->target_language),
          $oht_uuid,
          $job->settings['notes'],
          $job->settings['expertise']
        );

        $job_item->addRemoteMapping(NULL, $result['project_id'], array(
          'remote_identifier_2' => $oht_uuid,
          'word_count' => $result['wordcount'],
          'remote_data' => array(
            'credits' => $result['credits'],
          ),
        ));

        $job_item->addMessage('OHT Project ID %project_id created. @credits credits reduced from your account.',
          array('%project_id' => $result['project_id'], '@credits' => $result['credits']));
      }

      $job->submitted('Job has been successfully submitted for translation.');
    }
    catch (TMGMTOHTException $e) {
      $job->rejected($e->getMessage());
    }
  }

  /**
   * Returns list of expertise options.
   *
   * @param \TMGMTTranslator $translator
   *   TMGMTTranslator object.
   * @param string $source_language
   *   (optional) The source language. Mandatory if target language is
   *   specified.
   * @param string $target_language
   *   (optional) The target language. Mandatory if source language is
   *   specified.
   *
   * @return array
   *   List of expertise options, keyed by their code.
   */
  public function getExpertise(TMGMTTranslator $translator, $source_language = NULL, $target_language = NULL) {
    try {
      $expertise_codes = $this->getConnector($translator)->getExpertise($source_language, $target_language);
      $structured_expertise_codes = array();
      foreach ($expertise_codes as $expertise) {
        $structured_expertise_codes[$expertise['slug']] = $expertise['name'];
      }
      return $structured_expertise_codes;
    } catch (TMGMTOHTException $e) {
      return array();
    }
  }

  /**
   * Returns the list of account details.
   *
   * @param \TMGMTTranslator $translator
   *   TMGMTTranslator object.
   *
   * @return array|mixed
   *   The list of account details.
   */
  public function getAccountDetails(TMGMTTranslator $translator) {
    try {
      return $this->getConnector($translator)->getAccountDetails();
    } catch (TMGMTOHTException $e) {
      return array();
    }
  }

  /**
   * Returns quotation.
   *
   * @param TMGMTJob $job
   *   The job to get quotation for.
   *
   * @return array
   *   List of quotation value.
   */
  public function getQuotation(TMGMTJob $job) {
    $xliff_converter = tmgmt_file_format_controller('xlf');
    $translator = $job->getTranslator();
    $word_count = $job->getWordCount();
    $source_language = $job->source_language;
    $target_language = $job->target_language;
    $service = 'translation';
    $expertise = $job->getSetting('expertise');

    $name = "JobID" . $job->tjid . '_' . $job->source_language . '_' . $job->target_language;
    $connector = $this->getConnector($translator);
    try {
      // @todo: Move to getResourceUuids().
      $xliff = $xliff_converter->export($job, array());
      $resource_uuid = $connector->uploadFileResource($xliff, $name);
      return $connector->getQuotation($resource_uuid, $word_count, $source_language, $target_language, $service, $expertise);
      } catch (TMGMTOHTException $e) {
      return array();
    }
  }

  /**
   * Fetches translations for job items of a given job.
   *
   * @param TMGMTJob $job
   *   A job containing job items that translations will be fetched for.
   *
   * @return bool
   *   Returns TRUE if there are error messages during the process of retrieving
   *   translations. Otherwise FALSE.
   */
  public function fetchJobs(TMGMTJob $job) {
    // Search for placeholder item.
    $remotes = entity_get_controller('tmgmt_remote')->loadByLocalData($job->tjid);

    // Get job items of a given job.
    $job_items = $job->getItems();
    $translated = 0;
    $not_translated = 0;

    try {
      // Loop over job items and check for if there is a translation available.
      foreach ($remotes as $remote) {
        $job_item = $job_items[$remote->tjiid];
        if (empty($remote->remote_identifier_1)) {
          $not_translated++;
          $job_item->addMessage('Could not retrieve project information.', array(), 'error');
          continue;
        }
        $project_details = $this->getConnector($job->getTranslator())->getProjectDetails($remote->remote_identifier_1);
        // Retrieve translations.
        if (!empty($project_details['resources']['translations'])) {
          $this->retrieveTranslation((array) $project_details['resources']['translations'], $job_item, $project_details['project_id'], FALSE);
          $translated++;
        }
        elseif ($project_details['project_status_code'] == 'cancelled') {
          // Abort the job item.
          $job_item->setState(TMGMT_JOB_ITEM_STATE_ABORTED, 'Aborted by One Hour Translation.');
          $job_item->save();
          $not_translated++;
        }
        else {
          $not_translated++;
        }
      }

      if (empty($not_translated)) {
        $job->addMessage('Fetched translations for @translated job items.', array('@translated' => $translated));
      }
      else {
        $job->addMessage('Fetched translations for @translated job items, @not_translated are not translated yet.', array('@translated' => $translated, '@not_translated' => $not_translated));
      }
    } catch (TMGMTOHTException $e) {
      $job->addMessage('Could not pull translation resources.', array(), 'error');
    }
  }

  /**
   * Receives and stores a translation returned by OHT.
   */
  public function retrieveTranslation(array $resource_uuids, TMGMTJobItem $job_item, $oht_project_id = NULL, $add_message = TRUE) {
    try {
      foreach ($resource_uuids as $resource_uuid) {
        $resource = $this->getConnector($job_item->getTranslator())->getResourceDownload($resource_uuid, $oht_project_id);
        // On sandbox we get some non existing resource_uuids which do not
        // really exist and return some forbidden error in json format. So we
        // filter these out.
        if (strpos($resource, '<?xml') !== FALSE) {
          $data = $this->parseTranslationData($resource);
          $job_item->getJob()->addTranslatedData($data);

          if ($add_message) {
            if ($job_item->isState(TMGMT_JOB_ITEM_STATE_ACTIVE)) {
              $job_item->addMessage('The translation has been received.');
            }
            else {
              $job_item->addMessage('The translation has been updated.');
            }
          }
        }
        // @todo we should log errors here for the failing resources.
      }
    }
    catch (TMGMTOHTException $e) {
      $job_item->addMessage('Could not get translation from OHT. Message error: @error',
        array('@error' => $e->getMessage()), 'error');
    }
  }

  /**
   * Parses received translation from OHT and returns unflatted data.
   *
   * @param string $data
   *   Base64 encode data, received from OHT.
   *
   * @return array
   *   Unflatted data.
   */
  protected function parseTranslationData($data) {
    $xlf_converter = tmgmt_file_format_controller('xlf');
    // Import given data using XLIFF converter. Specify that passed content is
    // not a file.
    return $xlf_converter->import($data, FALSE);
  }

  /**
   * {@inheritdoc}
   */
  public function abortTranslation(TMGMTJob $job) {
    /** @var TMGMTRemoteController $remote_controller */
    $remote_controller = entity_get_controller('tmgmt_remote');
    $remotes = $remote_controller->loadByLocalData($job->tjid);

    // Loop over job items and cancel each of them.
    /** @var TMGMTRemote $remote */
    foreach ($remotes as $remote) {
      /** @var TMGMTJobItem $job_item */
      $job_item = $remote->getJobItem();
      if (!$job_item->isAborted()) {
        try {
          // Cancel the project on One Hour Translation site.
          $this->getConnector($job->getTranslator())->cancelProject($remote->remote_identifier_1);
          // Abort the job item.
          $job_item->setState(TMGMT_JOB_ITEM_STATE_ABORTED, 'Aborted by user.');
        }
        catch (TMGMTException $e) {
          $job_item->addMessage('Abortion failed: @error', array('@error' => $e->getMessage()), 'error');
        }
        $job_item->save();
      }
    }

    // Abort the job in the current system.
    return parent::abortTranslation($job);
  }

  /**
   * {@inheritdoc}
   */
  public function defaultSettings() {
    $defaults = parent::defaultSettings();
    // Enable CDATA for content encoding in File translator.
    $defaults['xliff_cdata'] = TRUE;
    return $defaults;
  }

  /**
   * Gets OHT service connector.
   *
   * @param TMGMTTranslator $translator
   *   Current job translator.
   *
   * @return OHTConnector
   *   OHT connector instance.
   */
  public function getConnector(TMGMTTranslator $translator) {
    return new OHTConnector(
      $translator->getSetting('api_public_key'),
      $translator->getSetting('api_secret_key'),
      $translator->getSetting('use_sandbox')
    );
  }
}
