<?php

/**
 * @file
 * Please supply a file description.
 */

/**
 * @file
 * Provides myGengo translation plugin controller.
 */
class TMGMTOhtTranslatorUIController extends TMGMTDefaultTranslatorUIController {

  /**
   * Implements TMGMTTranslatorUIControllerInterface::reviewForm().
   */
  public function reviewForm($form, &$form_state, TMGMTJobItem $job_item) {
    /* @var TMGMTOhtPluginController $controller */
    $controller = $job_item->getTranslatorController();
    $connector = $controller->getConnector($job_item->getTranslator());
    $mappings = $job_item->getRemoteMappings();
    /* @var TMGMTRemote $mapping */
    $mapping = array_shift($mappings);

    $comments = $connector->getProjectComments($mapping->remote_identifier_1);
    $rows = array();
    $new_comment_link = '';

    if (is_array($comments) && count($comments) > 0) {
      foreach ($comments as $comment) {
        $rows[] = array(
          array(
            'data' => t('By %name (%role) at %time', array(
              '%name' => $comment['commenter_name'],
              '%role' => $comment['commenter_role'],
              '%time' => format_date(strtotime($comment['date']))
            )),
            'class' => 'meta',
          ),
          check_plain($comment['comment_content']),
        );
      }

      $new_comment_link = '<a href="#new-comment">' . t('Add new comment') . '</a>';
    }

    $form['#attached']['css'][] = drupal_get_path('module', 'tmgmt_oht') . '/css/tmgmt_oht_comments.css';
    $form['#attached']['js'][] = drupal_get_path('module', 'tmgmt_oht') . '/js/tmgmt_oht_comments.js';

    $form['oht_comments'] = array(
      '#type' => 'fieldset',
      '#title' => t('OHT Comments'),
      '#collapsible' => TRUE,
    );
    $form['oht_comments']['container'] = array(
      '#prefix' => '<div id="tmgmt-oht-comments-wrapper">',
      '#suffix' => '</div>',
    );
    $form['oht_comments']['container']['comments'] = array(
      '#theme' => 'table',
      '#rows' => $rows,
      '#header' => array(),
      '#empty' => t('No comments'),
      '#prefix' => $new_comment_link,
    );
    $form['oht_comments']['container']['comment'] = array(
      '#type' => 'textarea',
      '#title' => t('Comment text'),
      '#prefix' => '<a name="new-comment"></a>'
    );
    $form['oht_comments']['container']['comment_submitted'] = array(
      '#type' => 'hidden',
      '#value' => isset($form_state['comment_submitted']) ? $form_state['comment_submitted'] : 0,
    );
    $form['oht_comments']['add_comment'] = array(
      '#type' => 'submit',
      '#value' => t('Add new comment'),
      '#submit' => array('tmgmt_oht_add_comment_form_submit'),
      '#ajax' => array(
        'callback' => 'tmgmt_oht_review_form_ajax',
        'wrapper' => 'tmgmt-oht-comments-wrapper',
      ),
    );

    return $form;
  }

  /**
   * Overrides TMGMTDefaultTranslatorUIController::pluginSettingsForm().
   */
  public function pluginSettingsForm($form, &$form_state, TMGMTTranslator $translator, $busy = FALSE) {
    $form['registration_link'] = array(
      '#type' => 'item',
      '#markup' => t('To support further development of this project, use the following link when registering for an account: <a href="@url" alt="OneHourTranslation">@url</a>.', array('@url' => 'http://www.onehourtranslation.com/affiliate/mirod')),
    );
    $form['api_public_key'] = array(
      '#type' => 'textfield',
      '#title' => t('OHT API Public Key'),
      '#default_value' => $translator->getSetting('api_public_key'),
      '#description' => t('Please enter your public API key or visit <a href="@url">the API keys page</a> to get one.', array('@url' => 'https://www.onehourtranslation.com/profile/apiKeys')),
    );
    $form['api_secret_key'] = array(
      '#type' => 'textfield',
      '#title' => t('OHT API Secret key'),
      '#default_value' => $translator->getSetting('api_secret_key'),
      '#element_validate' => array('tmgmt_oht_plugin_settings_form_validate'),
      '#description' => t('Please enter your secret API key or visit <a href="@url">the API keys page</a> to get one.', array('@url' => 'https://www.onehourtranslation.com/profile/apiKeys')),
    );
    $form['use_sandbox'] = array(
      '#type' => 'checkbox',
      '#title' => t('Use the sandbox'),
      '#default_value' => $translator->getSetting('use_sandbox'),
      '#description' => t('Check to use the testing environment.'),
    );
    return parent::pluginSettingsForm($form, $form_state, $translator, $busy);
  }

  /**
   * Implements TMGMTTranslatorUIControllerInterface::checkoutInfo().
   */
  public function checkoutInfo(TMGMTJob $job) {
    $form = array();

    if ($job->isActive()) {
      $form['actions']['pull'] = array(
        '#type' => 'submit',
        '#value' => t('Pull translations'),
        '#submit' => array('_tmgmt_oht_pull_submit'),
        '#weight' => -10,
      );
    }

    return $form;
  }

  /**
   * Overrides TMGMTDefaultTranslatorUIController::checkoutSettingsForm().
   */
  public function checkoutSettingsForm($form, &$form_state, TMGMTJob $job) {
    /**
     * @var TMGMTOhtPluginController $controller The translator object.
     */
    $controller = $job->getTranslator()->getController();
    $translator = $job->getTranslator();

    $settings['expertise'] = array(
      '#type' => 'select',
      '#title' => t('Expertise'),
      '#submit' => array('tmgmt_oht_checkout_form_expertise_update'),
      '#executes_submit_callback' => TRUE,
      '#ajax' => array(
        'callback' => 'tmgmt_oht_checkout_form_expertise_update_ajax',
        'wrapper' => 'tmgmt-ui-translator-settings',
      ),
      '#description' => t('Select an expertise to identify the area of the text you will request to translate.'),
      '#empty_option' => ' - ',
      '#options' => $controller->getExpertise($translator, $job->source_language, $job->target_language),
      '#default_value' => isset($job->settings['expertise']) ? $job->settings['expertise'] : '',
    );
    $settings['notes'] = array(
      '#type' => 'textarea',
      '#title' => t('Instructions'),
      '#description' => t('You can provide a set of instructions so that the translator will better understand your requirements.'),
      '#default_value' => isset($job->settings['notes']) ? $job->settings['notes'] : '',
    );
    $price_quote = $controller->getQuotation($job);
    $currency = $price_quote['currency'] == 'EUR' ? '€' : $price_quote['currency'];
    $total = $price_quote['total'];
    $settings['price_quote'] = array(
      '#type' => 'item',
      '#title' => t('Price quote'),
      '#markup' => t('<strong>@word_count</strong> words, <strong>@credits</strong> credits/<strong>@total_price@currency</strong>.', array(
        '@word_count' => $total['wordcount'],
        '@net_price' => $total['net_price'],
        '@transaction_fee' => $total['transaction_fee'],
        '@total_price' => $total['price'],
        '@credits' => $total['credits'],
        '@currency' => $currency,
      )),
    );
    $account_details = $controller->getAccountDetails($translator);
    $settings['account_balance'] = array(
      '#type' => 'item',
      '#title' => t('Account balance'),
      '#markup' => t('<strong>@credits</strong> credits', array('@credits' => $account_details['credits'])),
    );

    if ($account_details['credits'] < $total['price']) {
      $settings['low_account_balance'] = array(
        '#type' => 'item',
        '#markup' => t('Your account balance is lower than quoted price and the translation will not be successful.'),
        '#prefix' => '<div class="messages warning">',
        '#suffix' => '</div>',
      );
    }

    return $settings;
  }
}
